==> beagle-2.1.5.tar.gz: beagle-2.1.5/beagle/GA/src/InitBitStrOp.cpp <==
/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GA/src/InitBitStrOp.cpp
 *  \brief  Source code of class GA::InitBitStrOp.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.8 $
 *  $Date: 2004/07/04 16:58:22 $
 */

#include "beagle/GA.hpp"

#include <sstream>

using namespace Beagle;


/*!
 *  \brief Construct a GA bit string initialization operator.
 *  \param inBitStringSize Size of the bit strings initialized.
 *  \param inReproProbaName Reproduction probability parameter name used in register. 
 *  \param inName Name of the operator.
 */
GA::InitBitStrOp::InitBitStrOp(IntegerVector inBitStringSize,
                               std::string inReproProbaName,
                               std::string inName) :
  InitializationOp(inReproProbaName, inName),
  mBitStringSize(new IntegerVector(inBitStringSize)),
  mBitOneProba(NULL)
{ }


/*!
 *  \brief Initialize the GA bit string initialization operator.
 *  \param ioSystem System of the evolution.
 */
void GA::InitBitStrOp::initialize(System& ioSystem)
{
  InitializationOp::initialize(ioSystem);

  if(ioSystem.getRegister().isRegistered("ga.init.bitstrsize")) {
    mBitStringSize = castHandleT<IntegerVector>(ioSystem.getRegister()["ga.init.bitstrsize"]);
  } else {
    std::string lDefaultValue;
    for(unsigned int i=0; i<mBitStringSize->size(); i++) {
      lDefaultValue += uint2str((*mBitStringSize)[i]);
      if(i != (mBitStringSize->size()-1)) lDefaultValue += "/";
    }
    std::string lLongDescrip("Bit string size for individuals. The size of the ");
    lLongDescrip += "IntegerVector is the number of bit strings composing the GA individuals, ";
    lLongDescrip += "while each value of the vector is the number of bits composing each string. ";
    lLongDescrip += "IntegerVector format is S1/S2/.../Sn, where Si is ith value.";
    Register::Description lDescription(
      "Initial bit strings sizes",
      "IntegerVector",
      lDefaultValue,
      lLongDescrip
    );
    ioSystem.getRegister().addEntry("ga.init.bitstrsize", mBitStringSize, lDescription);
  }

  if(ioSystem.getRegister().isRegistered("ga.init.bitpb")) {
    mBitOneProba = castHandleT<Float>(ioSystem.getRegister()["ga.init.bitpb"]);
  } else {
    mBitOneProba = new Float(0.5);
    std::string lLongDescrip = "Distribution probability of bit values. ";
    lLongDescrip += "A probability of 1.0 means that the bits values are all initialized to 1, ";
    lLongDescrip += "while a probability of 0.0 means that they are all initialized to 0. ";
    lLongDescrip += "Probability of 0.5 means that the bits are uniformly, ";
    lLongDescrip += "randomly initialized with equally 0s and 1s.";
    Register::Description lDescription(
      "Initial bit distribution prob.",
      "Float",
      "0.5",
      lLongDescrip
    );
    ioSystem.getRegister().addEntry("ga.init.bitpb", mBitOneProba, lDescription);
  }
}


/*!
 *  \brief Initialize an individual made of GA bit strings.
 *  \param outIndividual Individual to initialize.
 *  \param ioContext Evolution context.
 */
void GA::InitBitStrOp::initIndividual(Beagle::Individual& outIndividual, Context& ioContext)
{
#ifndef BEAGLE_NDEBUG
  if(mBitStringSize->size() == 0) {
    std::string lMessage = "GA::InitBitStrOp::initIndividual: ";
    lMessage += "Size of the bit string size parameter is zero. ";
    lMessage += "Could not initialize the individuals!";
    throw Beagle_RunTimeExceptionM(lMessage);
  }
  for(unsigned int k=0; k<mBitStringSize->size(); k++) {
    if((*mBitStringSize)[k] == 0) {
      std::string lMessage = "GA::InitBitStrOp::initIndividual: Value of the ";
      lMessage += uint2ordinal(k);
      lMessage += " bit string size parameter is zero. ";
      lMessage += "Could not initialize the individuals!";
      throw Beagle_RunTimeExceptionM(lMessage);
    }
  }
#endif // BEAGLE_NDEBUG

  Beagle_LogVerboseM(
    ioContext.getSystem().getLogger(),
    "initialization", "Beagle::GA::InitBitStrOp",
    std::string("Resizing the individual to ")+
    uint2str(mBitStringSize->size())+" bitstrings"
  );
  outIndividual.resize(mBitStringSize->size());
  for(unsigned int i=0; i<outIndividual.size(); i++) {
    GA::BitString::Handle lActualBitString = castHandleT<GA::BitString>(outIndividual[i]);
    lActualBitString->resize((*mBitStringSize)[i]);
    for(unsigned int j=0; j<lActualBitString->size(); j++) {
      (*lActualBitString)[j] =
        (ioContext.getSystem().getRandomizer().rollUniform() < mBitOneProba->getWrappedValue());
    }
    Beagle_LogVerboseM(
      ioContext.getSystem().getLogger(),
      "initialization", "Beagle::GA::InitBitStrOp",
      std::string("Bitstring ")+uint2str(i+1)+
      std::string(" has ")+uint2str((*mBitStringSize)[i])+" initialized bits"
    );
    Beagle_LogDebugM(
      ioContext.getSystem().getLogger(),
      "initialization", "Beagle::GA::InitBitStrOp",
      uint2ordinal(i+1)+
      std::string(" bitstring is initialized as: ")+lActualBitString->serialize()
    );
  }
}

==> beagle-2.2.0.tar.gz: beagle-2.2.0/beagle/GA/src/InitBitStrOp.cpp <==
/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GA/src/InitBitStrOp.cpp
 *  \brief  Source code of class GA::InitBitStrOp.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.9 $
 *  $Date: 2004/10/25 20:45:32 $
 */

#include "beagle/GA.hpp"

#include <sstream>

using namespace Beagle;


/*!
 *  \brief Construct a GA bit string initialization operator.
 *  \param inNumberBits Number of bits of the strings initialized.
 *  \param inReproProbaName Reproduction probability parameter name used in register. 
 *  \param inName Name of the operator.
 */
GA::InitBitStrOp::InitBitStrOp(unsigned int inNumberBits,
                               std::string inReproProbaName,
                               std::string inName) :
  InitializationOp(inReproProbaName, inName),
  mNumberBits(new UInt(inNumberBits)),
  mBitOneProba(NULL)
{ }


/*!
 *  \brief Initialize the GA bit string initialization operator.
 *  \param ioSystem System of the evolution.
 */
void GA::InitBitStrOp::initialize(System& ioSystem)
{
  InitializationOp::initialize(ioSystem);

  if(ioSystem.getRegister().isRegistered("ga.init.numberbits")) {
    mNumberBits = castHandleT<UInt>(ioSystem.getRegister()["ga.init.numberbits"]);
  } else {
    std::string lLongDescrip("Number of bits used to initialize individuals.");
    Register::Description lDescription(
      "Initial number of bits in strings",
      "UInt",
      mNumberBits->serialize(),
      lLongDescrip
    );
    ioSystem.getRegister().addEntry("ga.init.numberbits", mNumberBits, lDescription);
  }

  if(ioSystem.getRegister().isRegistered("ga.init.bitpb")) {
    mBitOneProba = castHandleT<Float>(ioSystem.getRegister()["ga.init.bitpb"]);
  } else {
    mBitOneProba = new Float(0.5);
    std::string lLongDescrip = "Distribution probability of bit values. ";
    lLongDescrip += "A probability of 1.0 means that the bits values are all initialized to 1, ";
    lLongDescrip += "while a probability of 0.0 means that they are all initialized to 0. ";
    lLongDescrip += "Probability of 0.5 means that the bits are uniformly, ";
    lLongDescrip += "randomly initialized with equally 0s and 1s.";
    Register::Description lDescription(
      "Initial bit distribution prob.",
      "Float",
      "0.5",
      lLongDescrip
    );
    ioSystem.getRegister().addEntry("ga.init.bitpb", mBitOneProba, lDescription);
  }
}


/*!
 *  \brief Initialize an individual made of GA bit strings.
 *  \param outIndividual Individual to initialize.
 *  \param ioContext Evolution context.
 */
void GA::InitBitStrOp::initIndividual(Beagle::Individual& outIndividual, Context& ioContext)
{
#ifndef BEAGLE_NDEBUG
  if(mNumberBits->getWrappedValue() == 0) {
    std::string lMessage = "GA::InitBitStrOp::initIndividual: number of bits to use to ";
    lMessage += " initialize individuals parameter is zero. ";
    lMessage += "Could not initialize the individuals!";
    throw Beagle_RunTimeExceptionM(lMessage);
  }
#endif // BEAGLE_NDEBUG
  outIndividual.resize(1);
  GA::BitString::Handle lActualBitString = castHandleT<GA::BitString>(outIndividual[0]);
  lActualBitString->resize(mNumberBits->getWrappedValue());
  for(unsigned int i=0; i<lActualBitString->size(); ++i) {
    (*lActualBitString)[i] =
      (ioContext.getSystem().getRandomizer().rollUniform() < mBitOneProba->getWrappedValue());
  }
  Beagle_LogDebugM(
    ioContext.getSystem().getLogger(),
    "initialization", "Beagle::GA::InitBitStrOp",
    std::string("Bit string initialized as: ")+lActualBitString->serialize()
  );  
}
